;Ŀ
;                        Programmateur de PIC                                
;                                                                            
; Nom du fichier       : PROPIC0.ASM                                        
; Support physique     : ST62E15F1/HWD (version EPROM ou OTP)                
; Frquence d'horloge  : 4Mhz (rpl 13 cycles d'horloge pour 1 cycle machine) 
; Date de cration     : 09 / 03 / 96                                        
; Date de mise  jour  : 09 / 03 / 96                                        
; Auteur               : Michel OUSSET                                       
;

;                         Cablage des entres / sorties

;  PA0 sur OSC1        S / E
;  PA1 sur TOCKI       S / E    
;  PA2 sur Data In     E 
;  PA3 sur Data Out    S
;  PA4 sur Vpp         S
;  PA5 sur Vdd         S
;  PA6 sur Clk         E
;  PA7 sur NC          S
;  PB0 sur RB0         S / E
;  PB1 sur RB1         S / E
;  PB2 sur RB2         S / E
;  PB3 sur RB3         S / E
;  PB4 sur RB4         S / E
;  PB5 sur RB5         S / E
;  PB6 sur RB6         S / E
;  PB7 sur RB7         S / E
;  PC4 sur RA0         S / E
;  PC5 sur RA1         S / E
;  PC6 sur RA2         S / E
;  PC7 sur RA3         S / E




.VERS "ST6225"                 ;Dfinit la version du microprocesseur utilise
.ROMSIZE 4                     ;Taille de la ROM du microprocesseur
.PL 100                        ;100 lignes par page 
.TITLE "Programmateur de PIC"

;Dfinition de l'espace de donnes

X        .DEF  080H            ;Registre X (option 'm' pour le fichier DSD)
Y        .DEF  081H            ;Registre Y
V        .DEF  082H            ;Dfinition Registre V
W        .DEF  083H            ;Dfinition Registre W
A        .DEF  0FFH            ;Accumulateur

DRA      .DEF  0C0H            ;Registre de donne du port A (DATA Register)
DRB      .DEF  0C1H            ;Registre de donne du port B (DATA Register)
DRC      .DEF  0C2H            ;Registre de donne du port C (DATA Register)
DDRA     .DEF  0C4H            ;Registre de direction du port A (Data Direction Register)
DDRB     .DEF  0C5H            ;Registre de direction du port B (Data Direction Register)
DDRC     .DEF  0C6H            ;Registre de direction du port C (Data Direction Register)
ORA      .DEF  0CCH            ;Registre d'option du port A (Option Register Port)
ORB      .DEF  0CDH            ;Registre d'option du port B (Option Register Port)
ORC      .DEF  0CEH            ;Registre d'option du port C (Option Register Port)

IOR      .DEF  0C8H            ;Interrupt Option Register
DWR      .DEF  0C9H            ;Data ROM Window Register
ADR      .DEF  0D0H            ;A/D Data Register
ADCR     .DEF  0D1H            ;A/D Control Register
PSC      .DEF  0D2H            ;Timer PSC Register  (Timer Prescaler State Register)
TCR      .DEF  0D3H            ;Timer Data Register (Timer Counter Register)
TSCR     .DEF  0D4H            ;Timer TSCR Register (Timer Statut/Control Register)
WDR      .DEF  0D8H            ;Watchdog Register

;Dfinition des vecteurs d'interruption

.ORG     0FF0H                 ;Vecteur d'interruption #4 - A/D Interrupt
         RETI                  ;2 octets de disponibles
                              
.ORG     0FF2H                 ;Vecteur d'interruption #3 - Timer Interrupt
         JP    TINT            ;2 octets de disponibles

.ORG     0FF4H                 ;Vecteur d'interruption #2 - Port B & C Interrupt
         RETI                  ;2 octets de disponibles

.ORG     0FF6H                 ;Vecteur d'interruption #1 - Port A Interrupt
         RETI                  ;2 octets de disponibles

.ORG     0FFCH                 ;Vecteur d'interruption #0 - NMI Interrupt
         RETI                  ;2 octets de disponibles

.ORG     0FFEH                 ;Vecteur d'interruption ## - User RESET VECTOR
         JP    RESET           ;2 octets de disponibles

;Dclaration des constantes

DATAROMW .EQU  040H            ;Offset permettant l'accs  la DATA ROM Windows
CLK      .EQU  6               ;N bit d'entre du signal CLK
DIN      .EQU  2               ;N bit d'entre du signal Data In
DOUT     .EQU  3               ;N bit de sortie du signal Data Out

;Dclaration des variables (60 octets maxi)

.ORG     084H                  ;084H = 132, dbut de la zone de mmoire vive

COPA     .DEF  132             ;Copie du port A
COPB     .DEF  133             ;Copie du port B
COPC     .DEF  134             ;Copie du port C
CDE      .DEF  135             ;Rception de la commande
DATAL    .DEF  136             ;Poids faible donnes du PIC ou du PC
DATAH    .DEF  137             ;Poids fort donnes du PIC ou du PC

;Ŀ
;                           Macro d'initialisation du WATCHDOG               
;                                                                            
;       Temps de reset aprs initialisation : 1/4E6*12*2^8*2^6 = 49.152ms    
;

         .MACRO WATCH

         LDI    WDR,0FEH       ;Initialise le compteur WATCHDOG  1111 11 

         .ENDM

;Ŀ
;                           Programme principal                              
;

         .ORG  080H            ;User program ROM

RESET    RETI                  ;Fin de l'interruption RESET
         WATCH                 ;Initialise le WATCHDOG
         CALL  INIT            ;Initialisation du programme

R00      CALL  WSTART          ;Attente condition de start

         CLR   CDE             ;Initialisation de la commande

       JRR   3,COPA,WS2
       RES   3,COPA
       JP    WS3
WS2    SET   3,COPA
WS3    LD    A,COPA
       LD    DRA,A

                               ; Attente MSB == Bit 3 de la commande

         CALL  WBIT            ;Attente information d'entre
         JRS   0,V,R01         ;Un bit  t reu
         JP    R00             ;Une condition a t reue
R01      JRR   1,V,R02         ;Teste si bit  0
         SET   3,CDE           ;Positionne bit reu

                               ; Attente Bit 2 de la commande

R02      CALL  WBIT            ;Attente information d'entre
         JRS   0,V,R03         ;Un bit  t reu
         JP    R00             ;Une condition a t reue
R03      JRR   1,V,R04         ;Teste si bit  0
         SET   2,CDE           ;Positionne bit reu

                               ; Attente Bit 1 de la commande

R04      CALL  WBIT            ;Attente information d'entre
         JRS   0,V,R05         ;Un bit  t reu
         JP    R00             ;Une condition a t reue
R05      JRR   1,V,R06         ;Teste si bit  0
         SET   1,CDE           ;Positionne bit reu

                               ; Attente Bit 0 de la commande

R06      CALL  WBIT            ;Attente information d'entre
         JRS   0,V,R07         ;Un bit  t reu
         JP    R00             ;Une condition a t reue
R07      JRR   1,V,R08         ;Teste si bit  0
         SET   0,CDE           ;Positionne bit reu

                               ; Excution de la commande

R08      LD    A,CDE

         CPI   A,00            ; VDDP = ON (tension d'alimentation du PIC)
         JRNZ  R10
         CALL  CDE00           ;CDE00
         JP    R99

R10      CPI   A,01            ; VDDP = OFF
         JRNZ  R11
         CALL  CDE01           ;CDE01
         JP    R99

R11      CPI   A,02            ; VPP = ON (tension de programmation)
         JRNZ  R12
         CALL  CDE02           ;CDE02
         JP    R99

R12      CPI   A,03            ; VPP = ON
         JRNZ  R13
         CALL  CDE03           ;CDE03
         JP    R99

R13      CPI   A,04            ; TOCKI = 0 (validatoin du PIC en mode prog/vrif)
         JRNZ  R14
         CALL  CDE04           ;CDE04
         JP    R99

R14      CPI   A,05            ; TOCKI = 1 
         JRNZ  R15
         CALL  CDE05           ;CDE05
         JP    R99

R15      CPI   A,06            ; TOCKI = 0 pdt 10 000s (TPWF)
         JRNZ  R16
         CALL  CDE06           ;CDE06
         JP    R99

R16      CPI   A,07            ; TOCKI = 0 pdt 50s (TPWF)
         JRNZ  R17
         CALL  CDE07           ;CDE07
         JP    R99

R17      CPI   A,08            ; OSC1 = 0 (gestion du programm counter)
         JRNZ  R18
         CALL  CDE08           ;CDE08
         JP    R99

R18      CPI   A,09            ; OSC1 = 1 (gestion du programm counter)
         JRNZ  R19
         CALL  CDE09           ;CDE09
         JP    R99

R19      CPI   A,10            ; Port de donnes entre C et PIC en entre
         JRNZ  R20
         CALL  CDE10           ;CDE10
         JP    R99

R20      CPI   A,11            ; Port de donnes entre C et PIC en sortie
         JRNZ  R21
         CALL  CDE11           ;CDE11
         JP    R99

R21      CPI   A,12            ; Lire PIC et envoyer donnes au PC
         JRNZ  R22
         CALL  CDE12           ;CDE12
         JP    R99

R22      CPI   A,13            ; Lire PC et envoyer donnes au PIC
         JRNZ  R23
         CALL  CDE13           ;CDE13
         JP    R99

R23      CPI   A,14            ; Port de contrle entre C et PIC en entre
         JRNZ  R24
         CALL  CDE14           ;CDE14
         JP    R99

R24      CPI   A,15            ; Port de contrle entre C et PIC en sortie
         JRNZ  R25
         CALL  CDE15           ;CDE15
         JP    R99

R25
         

R99      JP    R00             ;Fin du programme principal

;Ŀ
;                                                                            
;                           Procdure de lecture du port d'entre            
;                                                                            
;       Condition de sortie : Information reue                              
;                                                                            
;       Sortie : Bit 0 (V) = 1 == bit reu, 0 == condition reue             
;                Bit 1 (V) = Bit reu                                        
;                Bit 2 (V) = Condition START                                 
;                Bit 3 (V) = Condition STOP                                  
;                                                                            
;

WBIT     CLR   V               ; Initialisation registre U

                               ; Dtection tat initial    
                               
         LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WB02      ;Teste si CLK = 0
         JRR   DIN,A,WB01      ;Teste si DIN = 0 (CLK = 1)

                               ; CLK = 1 & DIN = 1
                          
WB00     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WB02      ;Teste si CLK = 0 (front descendant de CLK)
         JRR   DIN,A,WB03      ;Teste si DIN = 0 (condition START)
         JP    WB00            ;Reboucle jusqu' changement d'tat

                               ; CLK = 1 & DIN = 0

WB01     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WB02      ;Teste si CLK = 0 (front descendant de CLK)
         JRS   DIN,A,WB04      ;Teste si DIN = 1 (conditon STOP)
         JP    WB01

                               ; CLK = 0 & DIN = x

WB02     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WB02      ;Rebouclage tant que CLK = 0

                               ;Front montant sur CLK

         SET   0,V             ;B0(U) = 1 (un bit reu)
         JRR   DIN,A,WB99      ;Teste si DIN = 0 => B1(U) = 0 ; bit reu = 0
         SET   1,V             ;B1(U) = 1 ; bit reu = 1
         JP    WB99

WB03     SET   2,V             ;Indicateur START positionn
         JP    WB99

WB04     SET   3,V             ;Indicateur STOP positionn
         JP    WB99

WB99     
       JRR   1,V,WS02
       SET   7,COPA
       JP    WS03
WS02   RES   7,COPA
WS03   LD    A,COPA
       LD    DRA,A

        RET

;Ŀ
;                                                                            
;                           Procdure attente du signal START                
;                                                                            
;       Condition de sortie : Signal START reu                              
;                                                                            
;

WSTART                         ; Attente condition de start

WS00     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WS00      ;Teste si CLK = 0
         JRR   DIN,A,WS00      ;Teste si DIN = 0 
         
                               ;CLK = 1 et DIN = 1

WS01     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WS00      ;Teste si CLK = 0
         JRS   DIN,A,WS01      ;Teste si DIN = 1

                               ;Front descendant sur DIN == Condition Start
         RET

;Ŀ
;                           Interruption Timer                               
;

TINT     RES   7,TSCR          ;Efface la demande d'interruption timer
         WATCH
         RETI

;Ŀ
;                           Initialisation du programme                      
;

INIT                           ; Initialisation des Entres / Sorties

         CALL  ESINIT
         
                               ; Initialisation des interruptions

         LDI   IOR,00010000B   ;Autorise les interruptions

                               ; Initialisation du Timer

         RES   3,TSCR          ;Initialise le prescaler  7F
         LDI   TSCR,01101101B  ;PS0, PS1, PS2 = 101 (facteur de division de 32)
                               ;Autorisation de comptage
                               ;DOUT, TOUT = Timer en sortie 0
                               ;Autorisation d'interruption

         
         RET                   ;Fin de la routine d'initialisation du programme

;Ŀ
;                           Initialisation des Entres/Sorties                                          
;                                                                                                       
; DDRA - DDRB - DDRC          (Direction)                                                               
;      ORA - ORB - ORC        (Option)                                                                  
;          DRA  - DRB  - DRC  (Data)                                                                    
; DDR  OR  DR   MODE    Option                                                                          
;  0   0   0    Input   With pull-up, no   interrupt (Etat aprs mise  zro)                           
;  0   0   1    Input   No   pull-up, no   interrupt                                                    
;  0   1   0    Input   With pull-up, with interrupt                                                    
;  0   1   1    Input   No   pull-up, no   interrupt, analog input (pour PA4-PA7, PB0-PB7, PC4-PC7 pin) 
;  1   0   x    Output  20mA sink Open-drain output (for the PA0-PA3 pins)                              
;                       Standard  Open-drain output (for the PA4-PA7, PB0-PB7, PC4-PC7 pins)            
;  1   1   x    Output  Push-pull output                                                                
;

ESINIT                         ; Initialisation du port A
         LDI   DDRA,10111000B                
         LDI   ORA, 10111000B  ;Pas de Pull Up pour les entres, sorties avec push pull
         LDI   COPA,11001111B
         LD    A,COPA          ;Recopie COPA dans DRA
         LD    DRA,A
                               ; Initialisation du port B
         LDI   DDRB,00000000B                
         LDI   ORB, 00000000B  ;Pas de Pull Up pour les entres, sorties avec push pull
         LDI   COPB,11111111B
         LD    A,COPB          ;Recopie COPB dans DRB
         LD    DRB,A
                               ; Initialisation du port C
         LDI   DDRC,00000000B                
         LDI   ORC, 00000000B  ;Pas de Pull Up pour les entres, sorties avec push pull
         LDI   COPC,11111111B
         LD    A,COPC          ;Recopie COPC dans DRC
         LD    DRC,A

         RET                   ;Retour

;Ŀ
;                                                                            
;                           Commande 00 : VDDP = ON                           
;                                                                            
;

CDE00    SET   5,COPA         ;Image du registre A
         LD    A,COPA         ;Commande de VDDP
         LD    DRA,A          ;Positionne la donne

         RET

;Ŀ
;                                                                            
;                           Commande 01 : VDDP = OFF                          
;                                                                            
;

CDE01    RES   5,COPA         ;Image du registre A
         LD    A,COPA         ;Commande de VDDP
         LD    DRA,A          ;Positionne la donne

         RET

;Ŀ
;                                                                            
;                           Commande 02 : VPP = ON                           
;                                                                            
;

CDE02    SET   4,COPA         ;Image du registre A
         LD    A,COPA         ;Commande de VPP
         LD    DRA,A          ;Positionne la donne

         RET

;Ŀ
;                                                                            
;                           Commande 03 : VPP = OFF                          
;                                                                            
;

CDE03    RES   4,COPA         ;Image du registre A
         LD    A,COPA         ;Commande de VPP
         LD    DRA,A          ;Positionne la donne

         RET

;Ŀ
;                                                                            
;                           Commande 04 : TOCKI = 0 (validation prog./vrif.)
;                                                                            
;

CDE04    RES   1,COPA         ;Image du registre A
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A

         RET

;Ŀ
;                                                                            
;                           Commande 05 : TOCKI = 1 (dvalidation prg./vr.) 
;                                                                            
;

CDE05    SET   1,COPA         ;Image du registre A
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A

         RET

;Ŀ
;                                                                            
;                           Commande 06 : TOCKI = 0 pdt 10 000s (TPWF)      
;                                                                            
; DOUT = 1 pendant la temporisation                                          
;                                                                            
;

CDE06    RES   1,COPA         ;Image du registre A (TOCKI = 0)
         SET   DOUT,COPA      ;Positionne DOUT  1
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A

         LDI   V,255          ;Temporisation de 10 000s
CD061    NOP
         NOP
         NOP
         DEC   V
         JRNZ  CD061

         SET   1,COPA         ;Image du registre A (TOCKI = 1)
         RES   DOUT,COPA      ;Positionne DOUT  0 (fin de tempo)  
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A

         RET

;Ŀ
;                                                                            
;                           Commande 07 : TOCKI = 0 pdt 50s (TPW)           
;                                                                            
;

CDE07    RES   1,COPA         ;Image du registre A (TOCKI = 0)
         LD    A,COPA         ;Positionne la donne en sortie  [4]
         LD    DRA,A

         NOP                  ;Temporisation de 50s           [2]
         NOP                  ;                                [2]

         SET   1,COPA         ;Image du registre A (TOCKI = 1) [4]
         LD    A,COPA         ;Positionne la donne en sortie  [4]
         LD    DRA,A

         RET

;Ŀ
;                                                                            
;                           Commande 08 : OSC1 = 0 (gestion prog. counter)   
;                                                                            
;

CDE08    RES   0,COPA         ;Image du registre A
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A          ;OSC1 = 0

         RET

;Ŀ
;                                                                            
;                           Commande 09 : OSC1 = 1 (gestion prog. counter)   
;                                                                            
;

CDE09    SET   0,COPA         ;Image du registre A
         LD    A,COPA         ;Positionne la donne en sortie
         LD    DRA,A          ;OSC1 = 1

         RET

;Ŀ
;                                                                            
;                           Commande 10 : Port de communication entre C et  
;                                         PIC en entre                      
;                                                                            
;

CDE10    LDI   ORB ,00000000B ;Port B sans interruption
         LDI   DDRB,00000000B ;Port B en entre
         LDI   COPB,11111111B ;Port B sans pull-up
         LDI   DRB ,11111111B

         LDI   ORC ,00000000B ;Port C sans interruption
         LDI   DDRC,00000000B ;Port C en entre
         LDI   COPC,11111111B ;Port C sans pull-up
         LDI   DRC ,11111111B

         RET

;Ŀ
;                                                                            
;                           Commande 11 : Port de communication entre C et  
;                                         PIC en sortie                      
;                                                                            
;

CDE11    CLR   COPB           ;Tous les bits du port B  0
         CLR   DRB            ;Recopie l'image du port B sur la sortie
         LDI   DDRB,11111111B ;Port B en sortie
         LDI   ORB ,11111111B ;Port B type push pull

         CLR   COPC           ;Tous les bits du port C  0
         CLR   DRC            ;Recopie l'image du port C sur la sortie
         LDI   DDRC,11111111B ;Port C en sortie
         LDI   ORC ,11111111B ;Port C type push pull

         RET

;Ŀ
;                                                                            
;                           Commande 12 : Lire PIC et envoyer donnes au PC  
;                                                                            
;       Sortie : Bit 0 (V) = 1 == 12 bits bien envoys                       
;                Bit 2 (V) = 1 == Condition START reue                      
;                Bit 3 (V) = 1 == Condition STOP reu                        
;                                                                            
;

CDE12    LD    A,DRC          ;Lecture du port C (poids faible)
         LD    DATAL,A        ;Stockage de la donne (bits 4  7)
         LD    A,DRB          ;Lecture du port B (poids fort)
         LD    DATAH,A        ;Stockage de la donne

                              ; Envoie bit 11

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1200      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1200    JRS   7,DATAH,C1201  ;Teste si bit 11 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1202
C1201    SET   DOUT,COPA      ;DOUT = 1
C1202    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 10

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1203      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1203    JRS   6,DATAH,C1204  ;Teste si bit 10 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1205
C1204    SET   DOUT,COPA      ;DOUT = 1
C1205    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 9

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1206      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1206    JRS   5,DATAH,C1207  ;Teste si bit 9 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1208
C1207    SET   DOUT,COPA      ;DOUT = 1
C1208    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 8

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1209      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1209    JRS   4,DATAH,C1210  ;Teste si bit 8 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1211
C1210    SET   DOUT,COPA      ;DOUT = 1
C1211    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 7

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1212      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1212    JRS   3,DATAH,C1213  ;Teste si bit 7 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1214
C1213    SET   DOUT,COPA      ;DOUT = 1
C1214    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 6

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1215      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1215    JRS   2,DATAH,C1216  ;Teste si bit 6 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1217
C1216    SET   DOUT,COPA      ;DOUT = 1
C1217    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 5

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1218      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1218    JRS   1,DATAH,C1219  ;Teste si bit 5 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1220
C1219    SET   DOUT,COPA      ;DOUT = 1
C1220    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 4

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1221      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1221    JRS   0,DATAH,C1222  ;Teste si bit 4 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1223
C1222    SET   DOUT,COPA      ;DOUT = 1
C1223    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 3

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1224      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1224    JRS   7,DATAL,C1225  ;Teste si bit 3 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1226
C1225    SET   DOUT,COPA      ;DOUT = 1
C1226    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 2

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1227      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1227    JRS   6,DATAL,C1228  ;Teste si bit 2 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1229
C1228    SET   DOUT,COPA      ;DOUT = 1
C1229    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 1

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1230      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1230    JRS   5,DATAL,C1231  ;Teste si bit 1 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1232
C1231    SET   DOUT,COPA      ;DOUT = 1
C1232    LD    A,COPA
         LD    DRA,A

                              ; Envoie bit 0

         CALL  WCLK           ;Attend front montant de l'horloge
                              
         JRS   1,V,C1233      ;Teste si front montant reu
         JP    C1299          ;Saut fin car condition reue
C1233    JRS   4,DATAL,C1234  ;Teste si bit 0 = 1
         RES   DOUT,COPA      ;DOUT = 0
         JP    C1235
C1234    SET   DOUT,COPA      ;DOUT = 1
C1235    LD    A,COPA
         LD    DRA,A

         CLR   V              ;12 bits bien envoys !
         SET   0,V

C1299    RET

;Ŀ
;                                                                            
;                           Commande 13 : Lire PC et envoyer donnes au PIC  
;                                                                            
;       Sortie : Bit 0 (V) = 1 == 12 bits bien reus                         
;                Bit 2 (V) = 1 == Condition START reue                      
;                Bit 3 (V) = 1 == Condition STOP reu                        
;                                                                            
;

CDE13    CLR   DATAL          ;Initialisation des variables de rception         
         CLR   DATAH

                              ; Rception bit 11

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1300      ;Teste si bit bien reu
         JP    C1399
C1300    JRR   1,V,C1301      ;Teste si bit reu = 0
         SET   7,DATAH

C1301                         ; Rception bit 10

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1302      ;Teste si bit bien reu
         JP    C1399
C1302    JRR   1,V,C1303      ;Teste si bit reu = 0
         SET   6,DATAH

C1303                         ; Rception bit 9

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1304      ;Teste si bit bien reu
         JP    C1399
C1304    JRR   1,V,C1305      ;Teste si bit reu = 0
         SET   5,DATAH

C1305                         ; Rception bit 8

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1306      ;Teste si bit bien reu
         JP    C1399
C1306    JRR   1,V,C1307      ;Teste si bit reu = 0
         SET   4,DATAH

C1307                         ; Rception bit 7

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1308      ;Teste si bit bien reu
         JP    C1399
C1308    JRR   1,V,C1309      ;Teste si bit reu = 0
         SET   3,DATAH

C1309                         ; Rception bit 6

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1310      ;Teste si bit bien reu
         JP    C1399
C1310    JRR   1,V,C1311      ;Teste si bit reu = 0
         SET   2,DATAH

C1311                         ; Rception bit 5

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1312      ;Teste si bit bien reu
         JP    C1399
C1312    JRR   1,V,C1313      ;Teste si bit reu = 0
         SET   1,DATAH

C1313                         ; Rception bit 4

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1314      ;Teste si bit bien reu
         JP    C1399
C1314    JRR   1,V,C1315      ;Teste si bit reu = 0
         SET   0,DATAH

C1315                         ; Rception bit 3

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1316      ;Teste si bit bien reu
         JP    C1399
C1316    JRR   1,V,C1317      ;Teste si bit reu = 0
         SET   7,DATAL

C1317                         ; Rception bit 2

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1318      ;Teste si bit bien reu
         JP    C1399
C1318    JRR   1,V,C1319      ;Teste si bit reu = 0
         SET   6,DATAL

C1319                         ; Rception bit 1

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1320      ;Teste si bit bien reu
         JP    C1399
C1320    JRR   1,V,C1321      ;Teste si bit reu = 0
         SET   5,DATAL

C1321                         ; Rception bit 0

         CALL  WBIT           ;Attend rception d'une information
         JRS   0,V,C1322      ;Teste si bit bien reu
         JP    C1399
C1322    JRR   1,V,C1323      ;Teste si bit reu = 0
         SET   4,DATAL

C1323    LDI   V,1            ;12 bits bien reus !

         LD    A,DATAH        ;Copie du poids fort sur le port
         LD    COPB,A
         LD    DRB,A
         LD    A,DATAL        ;Copie du poids faible sur le port
         LD    COPC,A
         LD    DRC,A
         
C1399    RET

;Ŀ
;                                                                            
;                           Commande 14 : Port de contrle entre C et PIC   
;                                         en entre                          
;                                                                            
;                                                                            
;

CDE14    SET   0,COPA         ;Place bit 0 et 1  1
         SET   1,COPA
         LDI   DDRA,10111000B ;Port A bits 0 et 1 en entre
         LDI   ORA ,10111000B ;Port A bits 0 et 1 sans interruption ni pull up
         LDI   A,COPA
         LD    DRA,A          ;Positionne le port  

         RET

;Ŀ
;                                                                            
;                           Commande 15 : Port de contrle entre C et PIC   
;                                         en sortie                          
;                                                                            
;

CDE15    RES   0,COPA         ;Place Bits 0 et 1  0
         RES   1,COPA
         LDI   DDRA,10111011B ;Port A bits 0 et 1 en sortie
         LDI   ORA ,10111011B ;Port A bits 0 et 1 type push pull
         LD    A,COPA
         LD    DRA,A          ;Positionne le port  

         RET

;Ŀ
;                                                                            
;                           Procdure d'attente du signal d'horloge          
;                                                                            
;       Condition de sortie : Signal reu                                    
;                                                                            
;       Sortie : Bit 1 (V) = CLK reu                                        
;                Bit 2 (V) = Condition START                                 
;                Bit 3 (V) = Condition STOP                                  
;                                                                            
;

WCLK     CLR   V               ; Initialisation registre U

                               ; Dtection tat initial    
                               
         LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WC02      ;Teste si CLK = 0
         JRR   DIN,A,WC01      ;Teste si DIN = 0 (CLK = 1)

                               ; CLK = 1 & DIN = 1
                          
WC00     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WC02      ;Teste si CLK = 0 (front descendant de CLK)
         JRR   DIN,A,WC03      ;Teste si DIN = 0 (condition START)
         JP    WC00            ;Reboucle jusqu' changement d'tat

                               ; CLK = 1 & DIN = 0

WC01     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WC02      ;Teste si CLK = 0 (front descendant de CLK)
         JRS   DIN,A,WC04      ;Teste si DIN = 1 (conditon STOP)
         JP    WC01

                               ; CLK = 0 & DIN = x

WC02     LD    A,DRA           ;Lecture port A
         JRR   CLK,A,WC02      ;Rebouclage tant que CLK = 0

         SET   1,V             ;Front montant sur CLK
         JP    WC99            

WC03     SET   2,V             ;Indicateur START positionn
         JP    WC99

WC04     SET   3,V             ;Indicateur STOP positionn

WC99     RET



.END
